// wrapper for midi fns in java


import javax.sound.midi.*;


public class MidiSynth {

   final int PROGRAM = 192;
   final int NOTEON = 144;
   final int NOTEOFF = 128;
   final int SUSTAIN = 64;
   final int REVERB = 91;
   final int ON = 0, OFF = 1;
   
   static Sequencer sequencer;
   static Sequence sequence;
   Synthesizer synthesizer;
   Instrument instruments[];
   MidiChannel[] channels;
   MidiChannel channel;


   String[] instrumentNames;

   int prevInst;
   int prevNote;

   int bufinst = 2;
   int bufnote = 60;
   int bufvelocity = 60;


   private String names[] = { 
      "Piano", "Chromatic Perc.", "Organ", "Guitar", 
      "Bass", "Strings", "Ensemble", "Brass", 
      "Reed", "Pipe", "Synth Lead", "Synth Pad",
      "Synth Effects", "Ethnic", "Percussive", "Sound Effects" };   

    Soundbank soundbank;
   public void open() {
      try {
	 if (synthesizer == null) {
	    if ((synthesizer = MidiSystem.getSynthesizer()) == null) {
	       System.out.println("getSynthesizer() failed!");
	       return;
	    }
	 } 
	 synthesizer.open();
	 sequencer = MidiSystem.getSequencer();
	 // sequence = new Sequence(Sequence.PPQ, 10);
	 sequencer.addMetaEventListener(new MetaEventListener()
	     {
		 public void meta(MetaMessage event)
		 {
		     System.out.println("Got a meta event");
		     if (event.getType() == 47)
			 {
			     sequencer.close();
			     if (synthesizer != null)
				 {
				     synthesizer.close();
				 }
			     System.exit(0);
			 }
		 }
	     });
	 

      } catch (Exception ex) { ex.printStackTrace(); return; }
      
      soundbank = synthesizer.getDefaultSoundbank();
      if (soundbank != null) {
	 instruments = synthesizer.getDefaultSoundbank().getInstruments();
	 synthesizer.loadInstrument(instruments[0]);

	 int ni = instruments.length;
	 instrumentNames = new String[ni];
	 for (int i = 0; i < ni; i++) {
	    instrumentNames[i] = instruments[i].getName();
	 }

      }
      channels = synthesizer.getChannels();
      channel = channels[0];
   }
   

   public void close() {
       System.out.println("Trying to close...");
       synthesizer.unloadAllInstruments(soundbank);
       /*
       if (synthesizer != null) {
	   synthesizer.close();
       }
       if (sequencer != null) {
	   sequencer.close();
	   }*/
       soundbank = null;
       sequencer = null;
       synthesizer = null;
       instruments = null;
       channel.allNotesOff();
       channels = null;
       System.gc();
       System.out.println("Closed...");
   }
 

   public void setInstrument(int program) {
      if (instruments != null) {
	 synthesizer.loadInstrument(instruments[program]);
      }
      channel.programChange(program);
   }


   public String[] getInstrumentNames() {
      if (synthesizer == null) open();
      return instrumentNames;
   }



   public void play(int inst, int note, int velocity) {
      bufinst = inst;
      bufnote = note;
      bufvelocity = velocity;

      if (synthesizer == null) open();
      if (channel == null) return;

      
      channel.noteOff(prevNote);
      
      
      if (inst != prevInst) {
	 synthesizer.loadInstrument(instruments[inst]);
	 channel.programChange(inst);
	 prevInst = inst;
      }

      channel.noteOn(note, velocity);
      prevNote = note;      

   }

   public void release() {
      channel.noteOff(prevNote);
   }

   public void release(int note, int velocity) {
      if (synthesizer == null) open();
      if (channel == null) return;
      channel.noteOff(note, velocity);
   }


   public void click() {
      play(bufinst, bufnote, bufvelocity);
   }

  
}

 
